@@ -8,10 +8,11 @@ import os
 from pprint import pformat
 
 from elftools.elf.elffile import ELFFile
+from elftools.elf.dynamic import DynamicSegment
 from elftools.common.exceptions import ELFError
 
 from .errors import *
-from .utils import coerce_sequence
+from .utils import coerce_sequence, single
 
 class ExternTool:
     def __init__(self, cmd, os_pkg, stderr_ignore=[], encoding=None):
@@ -70,7 +71,7 @@ class ExternTool:
 
 
 
-tool_ldd        = ExternTool('ldd', 'binutils')
+tool_ldd        = ExternTool(os.getenv("STATICX_LDD", "ldd"), 'binutils')
 tool_objcopy    = ExternTool('objcopy', 'binutils')
 tool_patchelf   = ExternTool('patchelf', 'patchelf',
                     stderr_ignore = [
@@ -129,17 +130,16 @@ def _parse_ldd_output(output):
         yield libpath
 
 
-def get_shobj_deps(path, libpath=[]):
+def get_shobj_deps(path, libpath=None):
     """Discover the dependencies of a shared object (*.so file)
     """
 
     # First verify we're dealing with a dynamic ELF file
     ensure_dynamic(path)
 
-
-    # TODO: Should we use dict(os.environ) instead?
-    #       For now, make sure we always pass a clean environment.
-    env = {}
+    # Prepare the environment
+    keep_vars = {'LD_LIBRARY_PATH'}
+    env = {k:v for k,v in os.environ.items() if k in keep_vars}
 
     if libpath:
         # Prepend to LD_LIBRARY_PATH
@@ -165,7 +165,10 @@ def get_shobj_deps(path, libpath=[]):
         # produced by musl-libc
         #
         # We simply raise a specific exception and let the caller deal with it.
-        raise LddError("Unexpected ldd error ({}): {}".format(rc, output))
+        message = "Unexpected ldd error ({}):\n{}".format(rc, output.strip('\n'))
+        if "invalid ELF header" in output:
+            message += "\nHint: try setting STATICX_LDD to the appropriate ldd for this executable"
+        raise LddError(message)
 
     return list(_parse_ldd_output(output))
 
@@ -194,6 +197,12 @@ def patch_elf(path, interpreter=None, rpath=None, force_rpath=False, no_default_
     if rpath:
         args += ['--set-rpath', rpath]
     if force_rpath:
+        # There is a bug in patchelf that requires --remove-rpath to be used
+        # first before --force-rpath is effective.
+        # https://github.com/NixOS/patchelf/issues/94
+        # This was fixed in v0.11 but that's newer than Debian Buster.
+        remove_rpath(path)
+
         args.append('--force-rpath')
     if add_needed:
         for lib in coerce_sequence(add_needed):
@@ -208,6 +217,8 @@ def patch_elf(path, interpreter=None, rpath=None, force_rpath=False, no_default_
     if no_default_lib:
         tool_patchelf.run_check('--no-default-lib', path)
 
+def remove_rpath(path):
+    tool_patchelf.run_check('--remove-rpath', path)
 
 def strip_elf(path):
     tool_strip.run_check(path)
@@ -223,57 +234,84 @@ class StaticELFError(Error):
         super().__init__(message)
 
 
-class ELFCloser:
-    def __init__(self, path, mode):
-        self.f = open(path, mode)
-        self.elf = ELFFile(self.f)
+class ELFFileX(ELFFile):
+    def __init__(self, stream, path=None):
+        self.__path = path
+        super().__init__(stream)
+
+    @classmethod
+    def open(cls, path, mode='rb'):
+        return cls(open(path, mode), path=path)
 
     def __enter__(self):
-        return self.elf
+        return self
 
     def __exit__(self, *exc_info):
-        self.f.close()
+        self.stream.close()
+
+    def get_single_section(self, sectype):
+        """Returns the only section of a given type, or None if absent"""
+        def key(sec):
+            return isinstance(sec, sectype)
+        return single(self.iter_sections(), key=key, default=None)
+
+    def get_prog_interp(self):
+        for seg in self.iter_segments():
+            # Amazingly, this is slightly faster than
+            # if isinstance(seg, InterpSegment):
+            try:
+                return seg.get_interp_name()
+            except AttributeError:
+                continue
+
+        raise InvalidInputError("{}: not a dynamic executable "
+                                "(no interp segment)".format(self.__path))
+
+
+    def get_dynamic_segment(self):
+        for seg in self.iter_segments():
+            if seg['p_type'] == 'PT_DYNAMIC':
+                assert isinstance(seg, DynamicSegment)
+                return seg
+        return None
+
+    def is_dynamic(self):
+        return bool(self.get_dynamic_segment())
+
+
+    def get_single_dynamic_tag(self, name):
+        dyn = self.get_dynamic_segment()
+        if dyn:
+            return single(dyn.iter_tags(name), default=None)
+        return None
+
+    def get_rpath(self):
+        """Returns the value of the DT_RPATH tag of the ELF file"""
+        return self.get_single_dynamic_tag('DT_RPATH')
+
+    def get_runpath(self):
+        """Returns the value of the DT_RUNPATH tag of the ELF file"""
+        return self.get_single_dynamic_tag('DT_RUNPATH')
+
 
 def open_elf(path, mode='rb'):
     try:
-        return ELFCloser(path, mode)
+        return ELFFileX.open(path, mode)
     except ELFError as e:
         raise InvalidInputError("{}: Invalid ELF image: {}".format(path, e))
 
 
-def get_section(elf, sectype):
-    for sec in elf.iter_sections():
-        if isinstance(sec, sectype):
-            return sec
-    return KeyError("Can't find section of type {}".format(sectype))
-
-
 def get_machine(path):
     with open_elf(path) as elf:
         return elf['e_machine']
 
 def get_prog_interp(path):
     with open_elf(path) as elf:
-        for seg in elf.iter_segments():
-            # Amazingly, this is slightly faster than
-            # if isinstance(seg, InterpSegment):
-            try:
-                return seg.get_interp_name()
-            except AttributeError:
-                continue
-        else:
-            raise InvalidInputError("{}: not a dynamic executable "
-                                    "(no interp segment)".format(path))
-
+        return elf.get_prog_interp()
 
 def is_dynamic(path):
     with open_elf(path) as elf:
-        for seg in elf.iter_segments():
-            if seg['p_type'] == 'PT_DYNAMIC':
-                # seg is an instance of DynamicSegment
-                return True
-        return False
-
+        return elf.is_dynamic()
 
 def ensure_dynamic(path):
     if not is_dynamic(path):
