@@ -2,10 +2,10 @@ import os
 import logging
 import tempfile
 
-from ..elf import get_shobj_deps, StaticELFError, LddError
+from ..elf import get_shobj_deps, is_dynamic, LddError
 from ..utils import make_executable, mkdirs_for
 
-def process_pyinstaller_archive(ctx):
+def process_pyinstaller_archive(sx):
     # See utils/cliutils/archive_viewer.py
 
     # If PyInstaller is not installed, do nothing
@@ -16,20 +16,20 @@ def process_pyinstaller_archive(ctx):
 
     # Attempt to open the program as PyInstaller archive
     try:
-        pyi_ar = CArchiveReader(ctx.orig_prog)
+        pyi_ar = CArchiveReader(sx.orig_prog)
     except:
         # Silence all PyInstaller exceptions here
         return
     logging.info("Opened PyInstaller archive!")
 
-    with PyInstallHook(ctx.archive, pyi_ar) as h:
+    with PyInstallHook(sx, pyi_ar) as h:
         h.process()
 
 
 
 class PyInstallHook:
-    def __init__(self, sx_archive, pyi_archive):
-        self.sx_ar = sx_archive
+    def __init__(self, sx, pyi_archive):
+        self.sx = sx
         self.pyi_ar = pyi_archive
 
         self.tmpdir = tempfile.TemporaryDirectory(prefix='staticx-pyi-')
@@ -79,17 +79,25 @@ class PyInstallHook:
     def _add_required_deps(self, lib):
         """Add dependencies of lib to staticx archive"""
 
-        # Silence "you do not have execution permission" warning from ldd
-        make_executable(lib)
-
-        # Assume this is a shared library, and
-        # try to get any dependencies of this file
-        try:
-            deps = get_shobj_deps(lib, libpath=[self.tmpdir.name])
-        except StaticELFError:
+        # Verify this is a shared library
+        if not is_dynamic(lib):
             # It's okay if there's a static executable in the PyInstaller
             # archive. See issue #78
             return
+
+        # Silence "you do not have execution permission" warning from ldd
+        make_executable(lib)
+
+        # Check for RPATH/RUNPATH, but only "dangerous" values and let
+        # "harmless" values pass (e.g. "$ORIGIN/cffi.libs")
+        self.sx.check_library_rpath(lib, dangerous_only=True)
+        # Unfortunately, there's no easy way to fix an UnsupportedRunpathError
+        # here, because staticx is not about to to modify the library and
+        # re-pack the PyInstaller archive itself.
+
+        # Try to get any dependencies of this file
+        try:
+            deps = get_shobj_deps(lib, libpath=[self.tmpdir.name])
         except LddError as e:
             # In certain cases, ldd might get upset about binary files
             # (probably added by the user via .spec file). This can happen
@@ -109,4 +117,4 @@ class PyInstallHook:
                 continue
 
             logging.debug("Adding {} to archive".format(dep))
-            self.sx_ar.add_library(deppath, exist_ok=True)
+            self.sx.add_library(deppath, exist_ok=True)
